#include "vtkapp.h"
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#define TERRAIN_COLOR_TEXTURE	0x01	//have color texture on, takes precedence over grey texture
#define TERRAIN_GREY_TEXTURE	0x02	//have grey texture on
#define TERRAIN_CONTOUR			0x04	//show contour levels
#define TERRAIN_USE_LOOKUP		0x08	//use lookup table
#define TERRAIN_USE_FANCYVIZ	0x10	//show fancy viz!!??



#define X 0
#define Y 1
#define Z 2
#define W 3

#define RED 0
#define GREEN 1
#define BLUE 2
#define YELLOW 3

///-----------------------------------
// VTK App messages supported

const int MyVTKApp::VTK_RENDER = 1;
const int MyVTKApp::VTK_LOAD_HIEGHTMAP = 2;
const int MyVTKApp::VTK_LOAD_TEXTURE = 3;
const int MyVTKApp::VTK_USE_GREY_TEXTURE = 4;
const int MyVTKApp::VTK_USE_COLOR_TEXTURE = 5;
const int MyVTKApp::VTK_CONTOURS_ON = 6;
const int MyVTKApp::VTK_CONTOURS_OFF = 7;
const int MyVTKApp::VTK_RED_CONTOUR = 8;
const int MyVTKApp::VTK_BLUE_CONTOUR = 9;
const int MyVTKApp::VTK_GREEN_CONTOUR = 10;
const int MyVTKApp::VTK_YELLOW_CONTOUR = 11;
const int MyVTKApp::VTK_USE_LOOKUP_TABLE = 12;
const int MyVTKApp::VTK_SCALE_HEIGHTMAP = 13;
const int MyVTKApp::VTK_UPDATE_HEIGHTMAP = 14;
const int MyVTKApp::VTK_DONT_USE_COLOR_TEXTURE = 15;
const int MyVTKApp::VTK_DONT_USE_FANCY_VIZ = 16;
const int MyVTKApp::VTK_DONT_USE_GREY_TEXTURE = 17;
const int MyVTKApp::VTK_USE_FANCY_VIZ = 18;
const int MyVTKApp::VTK_UPDATE_LOOKUP_TABLE = 19;
const int MyVTKApp::VTK_SHOW_HEIGHTMAP = 20;
const int MyVTKApp::VTK_HIDE_HEIGHTMAP = 21;


vtkLookupTable* colorTables[4] = {NULL, NULL, NULL, NULL};

MyVTKApp::MyVTKApp(HWND& hwnd, int width, int height):head(NULL),tail(NULL)
{
	parentWindow = hwnd;
	renderWindow = vtkRenderWindow::New();
	renderWindow->SetSize(width,height);
	renderWindow->SetParentId(hwnd);

	render1 = vtkRenderer::New();
	render1->SetBackground(0,0,0);

	interactor = vtkRenderWindowInteractor::New();

	renderWindow->AddRenderer(render1);
	interactor->SetRenderWindow(renderWindow);

	elevImgReader = vtkImageReader2::New();
	textureReader = vtkImageReader2::New();

	greyTexture = vtkTexture::New();
	colorTexture = vtkTexture::New();

	elevLut = vtkLookupTable::New();
	elevLut->SetNumberOfColors(32);
	elevLut->Build();
	
	elevLut->SetHueRange(0, 1.0);
	elevLut->SetSaturationRange(0, 1.0);
	elevLut->SetValueRange(0, 1.0);
	
	elevGeoFilter = vtkImageDataGeometryFilter::New();
	elevWarp = vtkWarpScalar::New();
	elevWarp->SetScaleFactor(-.5);
	geoGreyTextureMerge = vtkMergeFilter::New();
	geoColorTextureMerge = vtkMergeFilter::New();

	greyDataSetMapper = vtkDataSetMapper::New();
	colorDataSetMapper = vtkDataSetMapper::New();
	colorPolyMapper = vtkPolyDataMapper::New();
	colorPolyMapper->SetLookupTable(elevLut);
	colorPolyMapper->SetScalarRange(0,255);
	colorPolyMapper->SetColorModeToMapScalars();

	terrainLODActor = vtkLODActor::New();
	colorMapLODActor = vtkLODActor::New();
	colorMapLODActor->SetMapper(colorPolyMapper);
	
	

	terrainMode = 0;
	FancyOn = false;
	FancyCalculated = false;

	for(int i = 0; i < 4; i++)
	{
		contours[i] = vtkContourFilter::New();
		contours[i]->SetNumberOfContours(1);
		contours[i]->SetValue(0,0.0);
		contourMappers[i] = vtkPolyDataMapper::New();
		contourActors[i] = vtkLODActor::New();
		contourActors[i]->SetMapper(contourMappers[i]);
	
	}
	
	contourActors[RED]->GetProperty()->SetColor(1, 0, 0);
	contourActors[GREEN]->GetProperty()->SetColor(0, 1, 0);
	contourActors[BLUE]->GetProperty()->SetColor(0, 0, 1);
	contourActors[YELLOW]->GetProperty()->SetColor(1, 1, 0);

	
	for(int k = 0; k < 2; k++)
	{

			seeds[i] = NULL;
			streamers[i] = NULL;
			streamTubes[i] = NULL;
			streamTubeMappers[i] = NULL;
			streamActors[i] = NULL;
	}
	


	
}

MyVTKApp::~MyVTKApp()
{
	interactor->Delete();
	renderWindow->Delete();
	render1->Delete();
	
	elevImgReader->Delete();
	textureReader->Delete();
	greyTexture->Delete();
	colorTexture->Delete();
	elevGeoFilter->Delete();
	elevWarp->Delete();
	geoGreyTextureMerge->Delete();
	geoColorTextureMerge->Delete();
	greyDataSetMapper->Delete();
	colorDataSetMapper->Delete();
	terrainLODActor->Delete();
	colorPolyMapper->Delete();
	elevLut->Delete();
	colorMapLODActor->Delete();
	
	if(integrator)
		integrator->Delete();
	for(int i = 0; i < 4; i++)
	{
		if(contours[i])
			contours[i]->Delete();
		if(contourMappers[i])
			contourMappers[i]->Delete();
		if(contourActors[i])
			contourActors[i]->Delete();
	}

	for(int k = 0; k < 2; k++)
	{

		if(seeds[i])
			seeds[i]->Delete();
		if(streamers[i])
			streamers[i]->Delete();
		if(streamTubes[i])
			streamTubes[i]->Delete();
		if(streamTubeMappers[i])
			streamTubeMappers[i]->Delete();
		if(streamActors[i])
			streamActors[i]->Delete();
	}
	
}

void MyVTKApp::Render()
{

	if(renderWindow)
		renderWindow->Render();
}

void MyVTKApp::Message(VTKAppMessage* vtkAppMsg)
{
	VTKAppMessageQueue* newNode = new VTKAppMessageQueue();
	newNode->next = NULL;
	newNode->pMsgStruct = vtkAppMsg;
	//MessageBox(parentWindow,"Message Recieved","Acknowledgement",MB_OK);
	if(!head)
	{
		head = newNode;
		tail = newNode;
	}
	else
	{
		tail->next = newNode;
		tail = newNode;
		tail->next = NULL;
	}
}

void MyVTKApp::ProcessMessages()
{
	while(head)
	{
		VTKAppMessageQueue* temp;
		ImageFileData* imgData;
		LookupTableData* lutData;
		ContourData* contourData;
		char buff[32];
		float scaleFactor;
		float* scalarRange;
		int numberTemp;
		int extents[6];

		vtkPointSet* tempset;
		switch(head->pMsgStruct->msg)
		{
		case MyVTKApp::VTK_RENDER:					//////////////////////////////////
			if(renderWindow)
				renderWindow->Render();
			break;
		case MyVTKApp::VTK_LOAD_HIEGHTMAP:			//////////////////////////////////
			imgData = (ImageFileData*)(head->pMsgStruct->msgData);
//			MessageBox(parentWindow,"Press OK to load Height Map" ,"message",MB_OK);
			LoadGreyTexture(imgData);
			break;
		case MyVTKApp::VTK_LOAD_TEXTURE:			//////////////////////////////////
			imgData = (ImageFileData*)(head->pMsgStruct->msgData);
			LoadColorTexture(imgData);
			break;
		case MyVTKApp::VTK_UPDATE_HEIGHTMAP:		//////////////////////////////////
			SetupTerrain();
			break;
		case MyVTKApp::VTK_USE_LOOKUP_TABLE:		//////////////////////////////////
			render1->RemoveActor(terrainLODActor);
			render1->RemoveActor(colorMapLODActor);
			colorPolyMapper->Delete();
			colorPolyMapper = vtkPolyDataMapper::New();
			SetupColorMapTerrain();
			terrainMode = TERRAIN_USE_LOOKUP;

			break;
		case MyVTKApp::VTK_USE_COLOR_TEXTURE:		//////////////////////////////////
			render1->RemoveActor(terrainLODActor);
			render1->RemoveActor(colorMapLODActor);
			terrainLODActor->SetMapper(colorDataSetMapper);
			terrainLODActor->SetTexture(colorTexture);
			
			render1->AddActor(terrainLODActor);
			render1->ResetCamera();
			terrainMode = TERRAIN_COLOR_TEXTURE;
			break;
		case MyVTKApp::VTK_USE_GREY_TEXTURE:		//////////////////////////////////
			render1->RemoveActor(terrainLODActor);
			render1->RemoveActor(colorMapLODActor);
			
			terrainLODActor->SetMapper(greyDataSetMapper);
			terrainLODActor->SetTexture(greyTexture);
			
			render1->AddActor(terrainLODActor);
			render1->ResetCamera();

			terrainMode = TERRAIN_GREY_TEXTURE;
			break;
		case MyVTKApp::VTK_CONTOURS_OFF:			//////////////////////////////////
			for(numberTemp = 0; numberTemp < 4; numberTemp++)
			{
				if(contourActors[numberTemp])
				{
					render1->RemoveActor(contourActors[numberTemp]);
				}
			}

			renderWindow->Render();

			break;
		case MyVTKApp::VTK_CONTOURS_ON:				//////////////////////////////////
			ContourOn(RED);
//			MessageBox(parentWindow,"red done","contour",MB_OK);
			ContourOn(GREEN);
//			MessageBox(parentWindow,"green done","contour",MB_OK);
			ContourOn(BLUE);
//			MessageBox(parentWindow,"blue done","contour",MB_OK);
			ContourOn(YELLOW);
//			MessageBox(parentWindow,"yellow done","contour",MB_OK);
			break;
		case MyVTKApp::VTK_DONT_USE_FANCY_VIZ:		//////////////////////////////////
			FancyOn = false;
			renderWindow->StereoRenderOff();
			renderWindow->Render();
			break;
		case MyVTKApp::VTK_USE_FANCY_VIZ:			//////////////////////////////////
			FancyOn = true;
			
					renderWindow->SetStereoTypeToRedBlue();
					renderWindow->StereoRenderOn();
	
			renderWindow->Render();


			break;
		case MyVTKApp::VTK_UPDATE_LOOKUP_TABLE:		//////////////////////////////////
			lutData = (LookupTableData*) head->pMsgStruct->msgData;
			elevLut->SetHueRange(lutData->hueMin, lutData->hueMax);
			elevLut->SetSaturationRange(lutData->satMin, lutData->satMax);
			elevLut->SetValueRange(lutData->rngMin, lutData->rngMax);

			break;
		case MyVTKApp::VTK_HIDE_HEIGHTMAP:
			terrainLODActor->SetVisibility(0);
			colorMapLODActor->SetVisibility(0);
			break;
		case MyVTKApp::VTK_SHOW_HEIGHTMAP:
			colorMapLODActor->SetVisibility(1);
			terrainLODActor->SetVisibility(1);
			break;
		case MyVTKApp::VTK_RED_CONTOUR:
			contours[RED]->SetValue(0,(float)*((int*)(head->pMsgStruct->msgData)));
			//ContourOn(RED);
			break;
		case MyVTKApp::VTK_GREEN_CONTOUR:
			contours[GREEN]->SetValue(0,(float)*((int*)(head->pMsgStruct->msgData)));
			//ContourOn(GREEN);
			break;
		case MyVTKApp::VTK_BLUE_CONTOUR:
			contours[BLUE]->SetValue(0,(float)*((int*)(head->pMsgStruct->msgData)));
			//ContourOn(BLUE);
			break;
		case MyVTKApp::VTK_YELLOW_CONTOUR:
			contours[YELLOW]->SetValue(0,(float)*((int*)(head->pMsgStruct->msgData)));
			//ContourOn(YELLOW);
			break;
		case MyVTKApp::VTK_SCALE_HEIGHTMAP:
			scaleFactor = *((float*)(head->pMsgStruct->msgData));
			render1->RemoveActor(terrainLODActor);
			render1->RemoveActor(colorMapLODActor);

			terrainLODActor->Delete();
			colorMapLODActor->Delete();

			terrainLODActor = vtkLODActor::New();
			colorMapLODActor = vtkLODActor::New();

			//elevWarp->SetScaleFactor(scaleFactor);
			elevWarp->Delete();
			greyTexture->Delete();
			colorTexture->Delete();

			elevWarp = vtkWarpScalar::New();
			elevWarp->SetScaleFactor(scaleFactor);

			greyTexture = vtkTexture::New();
			colorTexture = vtkTexture::New();

			geoGreyTextureMerge->Delete();
			geoColorTextureMerge->Delete();

			geoGreyTextureMerge = vtkMergeFilter::New();
			geoColorTextureMerge = vtkMergeFilter::New();

			greyDataSetMapper->Delete();
			colorDataSetMapper->Delete();

			greyDataSetMapper = vtkDataSetMapper::New();
			colorDataSetMapper = vtkDataSetMapper::New();
			
			numberTemp = terrainMode;

			SetupTerrain();
			if(numberTemp == TERRAIN_GREY_TEXTURE)
			{	
	
				render1->RemoveActor(terrainLODActor);
				render1->RemoveActor(colorMapLODActor);
			
				terrainLODActor->SetMapper(greyDataSetMapper);
				terrainLODActor->SetTexture(greyTexture);
			
				render1->AddActor(terrainLODActor);
				render1->ResetCamera();

				terrainMode = TERRAIN_GREY_TEXTURE;
			}
			else if(numberTemp == TERRAIN_USE_LOOKUP)
			{
				SetupColorMapTerrain();
				render1->RemoveActor(terrainLODActor);
				SetupColorMapTerrain();
				terrainMode = TERRAIN_USE_LOOKUP;
			}
			else
			{
				render1->RemoveActor(colorMapLODActor);
				SetupTerrain();
			}

			//remove the contour actors and then put them back on so they rescale
			for(numberTemp = 0; numberTemp < 4; numberTemp++)
			{
				if(contourActors[numberTemp])
				{
					render1->RemoveActor(contourActors[numberTemp]);
				}
			}

			ContourOn(RED);
			ContourOn(GREEN);
			ContourOn(BLUE);
			ContourOn(YELLOW);

			break;
		default:									//////////////////////////////////

			break;
		};
		
		temp = head;
		head = head->next;
		delete temp;
	}

}


void MyVTKApp::LoadGreyTexture(ImageFileData* imgData)
{
	elevImgReader->SetFileName(imgData->fileName);
	elevImgReader->SetDataScalarTypeToUnsignedChar();
	elevImgReader->SetNumberOfScalarComponents(1);
	elevImgReader->SetDataExtent(0,imgData->width - 1, 0, imgData->height - 1, 0, 0);

}

void MyVTKApp::LoadColorTexture(ImageFileData* imgData)
{
	textureReader->SetFileName(imgData->fileName);
	textureReader->SetDataScalarTypeToUnsignedChar();
	textureReader->SetNumberOfScalarComponents(3);
	textureReader->SetDataExtent(0,imgData->width - 1, 0, imgData->height - 1, 0, 0);
}

void MyVTKApp::SetupTerrain()
{

	FancyOn = false;
	FancyCalculated = false;
	for(int i = 0; i < 2; i++)
	{
		if(streamActors[i])
			render1->RemoveActor(streamActors[i]);
	}

	terrainMode = TERRAIN_COLOR_TEXTURE;
	int	extents[6];
	elevImgReader->GetDataExtent(extents);
	elevGeoFilter->SetInput(elevImgReader->GetOutput());	//convert image data to geometry

	elevWarp->SetInput((vtkPointSet*) elevGeoFilter->GetOutput());			//pass geometry to warp scalar

	greyTexture->SetInput(elevImgReader->GetOutput());
	greyTexture->RepeatOff();

	colorTexture->SetInput(textureReader->GetOutput());
	colorTexture->RepeatOff();

	geoGreyTextureMerge->SetInput((vtkDataSet*) elevWarp->GetOutput());
	geoGreyTextureMerge->SetScalars((vtkDataSet*) elevImgReader->GetOutput());

	geoColorTextureMerge->SetInput((vtkDataSet*) elevWarp->GetOutput());
	geoColorTextureMerge->SetScalars((vtkDataSet*) textureReader->GetOutput());

	greyDataSetMapper->SetInput(geoGreyTextureMerge->GetOutput());
	greyDataSetMapper->SetScalarRange(0,255);
	greyDataSetMapper->ImmediateModeRenderingOff();
	colorDataSetMapper->SetInput(geoColorTextureMerge->GetOutput());
	colorDataSetMapper->SetScalarRange(0,255);
	colorDataSetMapper->ImmediateModeRenderingOff();

	terrainLODActor->SetMapper(colorDataSetMapper);
	terrainLODActor->SetTexture(colorTexture);
	terrainLODActor->SetNumberOfCloudPoints(extents[1] * extents[3] / 5);

	for(int i = 0; i < 4; i++)
	{
		contours[i]->SetInput((vtkDataSet*) elevGeoFilter->GetOutput());
	}
	render1->AddActor(terrainLODActor);
	render1->ResetCamera();

}


void MyVTKApp::SetupColorMapTerrain(void)
{
	if(!elevWarp->GetOutput())
		MessageBox(parentWindow,"No output from elev warp", "blah", MB_OK);

	colorPolyMapper->SetInput((vtkPolyData*)elevWarp->GetOutput());
	colorPolyMapper->SetLookupTable(elevLut);
	colorPolyMapper->SetScalarRange(0,255);
	colorPolyMapper->SetColorModeToMapScalars();

	int extents[6];
	elevImgReader->GetDataExtent(extents);
	colorMapLODActor->SetMapper(colorPolyMapper);
	colorMapLODActor->SetNumberOfCloudPoints( extents[1] * extents[3] / 5.0);
	
	render1->AddActor(colorMapLODActor);
	render1->RemoveActor(terrainLODActor);
	render1->ResetCamera();


}

void MyVTKApp::ResetTerrain(void)
{
}

void MyVTKApp::ResetColorMapTerrain(void)
{
}

void MyVTKApp::ContourOn(int color)
{

	static vtkWarpScalar* warp[4] = {NULL, NULL, NULL, NULL};
	if(colorTables[color] == NULL)
		colorTables[color] = vtkLookupTable::New();

	switch(color)
	{
	case RED:
		colorTables[color]->SetNumberOfTableValues(1);
		colorTables[color]->SetTableValue(0,1,0,0);
		break;
	case GREEN:
		colorTables[color]->SetNumberOfTableValues(1);
		colorTables[color]->SetTableValue(0,0,1,0);
		break;
	case BLUE:
		colorTables[color]->SetNumberOfTableValues(1);
		colorTables[color]->SetTableValue(0,0,0,1);
		break;
	case YELLOW:
		colorTables[color]->SetNumberOfTableValues(1);
		colorTables[color]->SetTableValue(0,1,1,0);
		break;
	}

		if(!contours[color]->GetInput())
			contours[color]->SetInput((vtkDataSet*) elevGeoFilter->GetOutput());

		contours[color]->SetComputeNormals(1);

		if(warp[color])
			warp[color]->Delete();
        
		warp[color] = vtkWarpScalar::New();

		warp[color]->SetInput((vtkPointSet*) contours[color]->GetOutput());		
		warp[color]->SetScaleFactor( elevWarp->GetScaleFactor());

		if(contourMappers[color])
			contourMappers[color]->Delete();

		contourMappers[color] = vtkPolyDataMapper::New();
		contourMappers[color]->SetInput((vtkPolyData*)warp[color]->GetOutput());
		contourMappers[color]->SetScalarRange(0,255);
		contourMappers[color]->SetLookupTable(colorTables[color]);
		contourMappers[color]->SetColorModeToMapScalars();

		if(contourActors[color])
			contourActors[color]->Delete();

		contourActors[color] = vtkLODActor::New();
		contourActors[color]->SetMapper(contourMappers[color]);
		render1->AddActor(contourActors[color]);

		contourActors[color]->SetNumberOfCloudPoints(1000);


	switch(color)
	{
	case RED:
		break;
	case GREEN:
		break;
	case BLUE:
		break;
	case YELLOW:
		break;
	}
}

