#ifndef VTK_APP_H
#define VTK_APP_H

#include <vtk/vtkProperty.h>
#include <vtk/vtkRenderWindow.h>
#include <vtk/vtkRenderer.h>
#include <vtk/vtkRenderWindowInteractor.h>
#include <vtk/vtkConeSource.h>
#include <vtk/vtkPolyDataMapper.h>
#include <vtk/vtkActor.h>

//--------------------------------------
#include <vtk/vtkImageReader2.h>
#include <vtk/vtkWarpScalar.h>
#include <vtk/vtkTexture.h>
#include <vtk/vtkMergeFilter.h>
#include <vtk/vtkImageDataGeometryFilter.h>
#include <vtk/vtkDataSetMapper.h>
#include <vtk/vtkLODActor.h>
#include <vtk/vtkLookupTable.h>

#include <vtk/vtkContourFilter.h>

#include <vtk/vtkPointSource.h>
#include <vtk/vtkRungeKutta2.h>
#include <vtk/vtkStreamLine.h>
#include <vtk/vtkTubeFilter.h>
#include <vtk/vtkPointSet.h>
#include <vtk/vtkStructuredGrid.h>
#include <vtk/vtkPointData.h>

#include <vtk/vtkCardinalSpline.h>
#include <vtk/vtkPoints.h>
#include <vtk/vtkMath.h>


//--------------------------------------
struct ImageFileData
{
	char* fileName;
	int width;
	int height;
};

struct ContourData
{
	int color;
	int level;
};

struct LookupTableData
{
	float hueMin;
	float hueMax;
	float satMin;
	float satMax;
	float rngMin;
	float rngMax;
};

struct VTKAppMessage
{
	int	msg;
	void* msgData;
};

struct VTKAppMessageQueue
{
	VTKAppMessage* pMsgStruct;
	VTKAppMessageQueue* next;
};

//		VTK APPLICATION CLASS
//		- Change the Render method to what ever way you want
//----------------------------------
class MyVTKApp 
{
public:
	MyVTKApp(HWND&, int, int);
	virtual ~MyVTKApp();

	virtual void	Render();
	void	Message(VTKAppMessage* );	//recieve messages
	void	ProcessMessages();			//process messages in message queue
	///-----------------------------------
	// VTK App messages supported
	static const int VTK_RENDER;
	static const int VTK_LOAD_HIEGHTMAP;
	static const int VTK_LOAD_TEXTURE;
	static const int VTK_UPDATE_HEIGHTMAP;
	static const int VTK_USE_GREY_TEXTURE;
	static const int VTK_USE_COLOR_TEXTURE;
	static const int VTK_SCALE_HEIGHTMAP;

	static const int VTK_CONTOURS_ON;
	static const int VTK_CONTOURS_OFF;
	static const int VTK_RED_CONTOUR;
	static const int VTK_GREEN_CONTOUR;
	static const int VTK_BLUE_CONTOUR;
	static const int VTK_YELLOW_CONTOUR;
	static const int VTK_USE_FANCY_VIZ;

	static const int VTK_USE_LOOKUP_TABLE;
	static const int VTK_DONT_USE_LOOKUP_TABLE;
	static const int VTK_DONT_USE_GREY_TEXTURE;
	static const int VTK_DONT_USE_COLOR_TEXTURE;
	static const int VTK_DONT_USE_FANCY_VIZ;
	static const int VTK_UPDATE_LOOKUP_TABLE;
	static const int VTK_HIDE_HEIGHTMAP;
	static const int VTK_SHOW_HEIGHTMAP;


private:

	bool	Updated();
	
	HWND parentWindow;
	int terrainMode;

	VTKAppMessageQueue *head;
	VTKAppMessageQueue *tail;

	vtkRenderer		*render1;
	vtkRenderWindow	*renderWindow;
	vtkRenderWindowInteractor	*interactor;

	vtkImageReader2	*elevImgReader;
	vtkImageReader2	*textureReader;

	vtkTexture		*greyTexture;
	vtkTexture		*colorTexture;

	vtkLookupTable	*elevLut;

	vtkImageDataGeometryFilter	*elevGeoFilter;
	vtkWarpScalar	*elevWarp;
	
	vtkMergeFilter	*geoGreyTextureMerge;
	vtkMergeFilter	*geoColorTextureMerge;

	vtkDataSetMapper	*greyDataSetMapper;
	vtkDataSetMapper	*colorDataSetMapper;
	vtkPolyDataMapper	*colorPolyMapper;

	vtkLODActor		*terrainLODActor;
	vtkLODActor		*colorMapLODActor;

	//Contour 
	vtkContourFilter	*contours[4];
	vtkPolyDataMapper	*contourMappers[4];
	vtkLODActor			*contourActors[4];

	bool FancyOn;
	bool FancyCalculated;
	
	vtkPointSource		*seeds[2];	//2 seed areas
	vtkRungeKutta2		*integrator;
	vtkStreamLine		*streamers[2];	//2 streamers
	vtkTubeFilter		*streamTubes[2];
	vtkPolyDataMapper	*streamTubeMappers[2];
	vtkLODActor			*streamActors[2];
	vtkLookupTable		*streamLut[2];
	vtkStructuredGrid	*streamGrid;

	//camera motion

//	PRIVATE METHODS

	void LoadGreyTexture(ImageFileData*);
	void LoadColorTexture(ImageFileData*);
	void SetupTerrain();
	void SetupColorMapTerrain(void);
	void ResetTerrain(void);
	void ContourOn(int);



public:
	void ResetColorMapTerrain(void);

};






#endif